package com.steerinfo.ems.cordasconfiguration.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.cordasconfiguration.model.CordasConfiguration;
import com.steerinfo.ems.cordasconfiguration.service.ICordasConfigurationService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * CordasConfiguration RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-12-23 10:30
 * 类描述
 * 修订历史：
 * 日期：2019-12-23
 * 作者：generator
 * 参考：
 * 描述：CordasConfiguration RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/cordasconfigurations")
public class CordasConfigurationController extends BaseRESTfulController {

    @Autowired
    ICordasConfigurationService cordasConfigurationService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("cordasconfiguration:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<CordasConfiguration> list = cordasConfigurationService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("cordasconfiguration:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap<String, Object> parmas,Integer pageNum, Integer pageSize){
        if(parmas.containsKey("workproc") && parmas.get("workproc") != null && !"".equals(parmas.get("workproc").toString())){
            String workprocs = parmas.get("workproc").toString();
            if(workprocs.indexOf(",") == -1){
                parmas.put("workproc", "and LOCATEID = '" + workprocs + "'");
            }
            else {
                workprocs = "and LOCATEID in ('" + workprocs.replaceAll(",", "','") + "')";
                parmas.put("workproc", workprocs);
            }
        }
        if(parmas.containsKey("energy") && parmas.get("energy") != null && !"".equals(parmas.get("energy").toString())){
            String energys = parmas.get("energy").toString();
            if(energys.indexOf(",") == -1){
                parmas.put("energy", "and ENERGYID = '" + energys + "'");
            }
            else {
                energys = "and ENERGYID in ('" + energys.replaceAll(",", "','") + "')";
                parmas.put("energy", energys);
            }
        }
        if(parmas.containsKey("property") && parmas.get("property") != null && !"".equals(parmas.get("property").toString())){
            String propertys = parmas.get("property").toString();
            if(propertys.indexOf(",") == -1){
                parmas.put("property", "and PROPERTYID = '" + propertys + "'");
            }
            else {
                propertys = "and PROPERTYID in ('" + propertys.replaceAll(",", "','") + "')";
                parmas.put("property", propertys);
            }
        }
        if(parmas.containsKey("energytype") && parmas.get("energytype") != null && !"".equals(parmas.get("energytype").toString())){
            String energytypes = parmas.get("energytype").toString();
            if(energytypes.indexOf(",") == -1){
                parmas.put("energytype", "and ENERGYTYPEID = '" + energytypes + "'");
            }
            else {
                energytypes = "and ENERGYTYPEID in ('" + energytypes.replaceAll(",", "','") + "')";
                parmas.put("energytype", energytypes);
            }
        }
        // PageList<CordasConfiguration> list = cordasConfigurationService.queryLikeForPage(parmas, pageNum, pageSize);
        PageList<Map<String, Object>> list = cordasConfigurationService.queryDataForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据CordasConfiguration对象创建")
    @ApiImplicitParam(name = "cordasConfiguration", value = "详细实体cordasConfiguration", required = true, dataType = "CordasConfiguration")
    //@RequiresPermissions("cordasconfiguration:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute CordasConfiguration model){
        CordasConfiguration cordasConfiguration = cordasConfigurationService.add(model);
        return success(cordasConfiguration);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("cordasconfiguration:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        CordasConfiguration cordasConfiguration = cordasConfigurationService.getById(id);
        return success(cordasConfiguration);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的cordasConfiguration信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "cordasConfiguration", value = "详细实体cordasConfiguration", required = true, dataType = "CordasConfiguration")
    })
    //@RequiresPermissions("cordasconfiguration:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody CordasConfiguration model){
        model.setId(id);
        CordasConfiguration cordasConfiguration = cordasConfigurationService.modify(model);
        return success(cordasConfiguration);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("cordasconfiguration:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> ids = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(ids)) {
			cordasConfigurationService.delete(ids);
    	}
        return success();
    }
}
