package com.steerinfo.ems.dispatchelog.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.dispatchelog.model.DispatchELog;
import com.steerinfo.ems.dispatchelog.service.IDispatchELogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * DispatchELog RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2020-08-08 11:39
 * 类描述
 * 修订历史：
 * 日期：2020-08-08
 * 作者：generator
 * 参考：
 * 描述：DispatchELog RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/dispatchelogs")
public class DispatchELogController extends BaseRESTfulController {

    @Autowired
    IDispatchELogService dispatchELogService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchelog:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchELog> list = dispatchELogService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchelog:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchELog> list = dispatchELogService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据DispatchELog对象创建")
    @ApiImplicitParam(name = "dispatchELog", value = "详细实体dispatchELog", required = true, dataType = "DispatchELog")
    //@RequiresPermissions("dispatchelog:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute DispatchELog model){
        DispatchELog dispatchELog = dispatchELogService.add(model);
        return success(dispatchELog);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchelog:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        DispatchELog dispatchELog = dispatchELogService.getById(id);
        return success(dispatchELog);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的dispatchELog信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "dispatchELog", value = "详细实体dispatchELog", required = true, dataType = "DispatchELog")
    })
    //@RequiresPermissions("dispatchelog:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody DispatchELog model){
    	if (model==null || model.getClock()==null || model.getClock().isEmpty()){
    		return failed(null,"请填写日志编号");
    	}
    	DispatchELog t = dispatchELogService.getById(model.getClock());
    	DispatchELog dispatchPLog = null;
    	if(t!=null){
    		dispatchPLog = dispatchELogService.modify(model);
    	}else{
    		dispatchPLog = dispatchELogService.add(model);
    	}
        return success(dispatchPLog);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchelog:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  dispatchELogService.delete(ids);
    	}
      return success();
    }
}
