package com.steerinfo.ems.dispatchwlog.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.dispatchwlog.model.DispatchWLog;
import com.steerinfo.ems.dispatchwlog.service.IDispatchWLogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * DispatchWLog RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2020-08-08 11:44
 * 类描述
 * 修订历史：
 * 日期：2020-08-08
 * 作者：generator
 * 参考：
 * 描述：DispatchWLog RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/dispatchwlogs")
public class DispatchWLogController extends BaseRESTfulController {

    @Autowired
    IDispatchWLogService dispatchWLogService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchwlog:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchWLog> list = dispatchWLogService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchwlog:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchWLog> list = dispatchWLogService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据DispatchWLog对象创建")
    @ApiImplicitParam(name = "dispatchWLog", value = "详细实体dispatchWLog", required = true, dataType = "DispatchWLog")
    //@RequiresPermissions("dispatchwlog:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute DispatchWLog model){
        DispatchWLog dispatchWLog = dispatchWLogService.add(model);
        return success(dispatchWLog);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchwlog:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        DispatchWLog dispatchWLog = dispatchWLogService.getById(id);
        return success(dispatchWLog);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的dispatchWLog信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "dispatchWLog", value = "详细实体dispatchWLog", required = true, dataType = "DispatchWLog")
    })
    //@RequiresPermissions("dispatchwlog:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody DispatchWLog model){
    	if (model==null || model.getClock()==null || model.getClock().isEmpty()){
    		return failed(null,"请填写日志编号");
    	}
    	DispatchWLog t = dispatchWLogService.getById(model.getClock());
    	DispatchWLog dispatchWLog = null;
    	if(t!=null){
    		dispatchWLog = dispatchWLogService.modify(model);
    	}else{
    		dispatchWLog = dispatchWLogService.add(model);
    	}
        return success(dispatchWLog);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchwlog:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  dispatchWLogService.delete(ids);
    	}
      return success();
    }
}
