package com.steerinfo.ems.dispatchzlog.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.dispatchzlog.model.DispatchZLog;
import com.steerinfo.ems.dispatchzlog.service.IDispatchZLogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * DispatchZLog RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2020-08-08 11:39
 * 类描述
 * 修订历史：
 * 日期：2020-08-08
 * 作者：generator
 * 参考：
 * 描述：DispatchZLog RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/dispatchzlogs")
public class DispatchZLogController extends BaseRESTfulController {

    @Autowired
    IDispatchZLogService dispatchZLogService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchzlog:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchZLog> list = dispatchZLogService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("dispatchzlog:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<DispatchZLog> list = dispatchZLogService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据DispatchZLog对象创建")
    @ApiImplicitParam(name = "dispatchZLog", value = "详细实体dispatchZLog", required = true, dataType = "DispatchZLog")
    //@RequiresPermissions("dispatchzlog:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute DispatchZLog model){
        DispatchZLog dispatchZLog = dispatchZLogService.add(model);
        return success(dispatchZLog);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchzlog:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        DispatchZLog dispatchZLog = dispatchZLogService.getById(id);
        return success(dispatchZLog);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的dispatchZLog信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "dispatchZLog", value = "详细实体dispatchZLog", required = true, dataType = "DispatchZLog")
    })
    //@RequiresPermissions("dispatchzlog:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody DispatchZLog model){
    	if (model==null || model.getClock()==null || model.getClock().isEmpty()){
    		return failed(null,"请填写日志编号");
    	}
    	DispatchZLog t = dispatchZLogService.getById(model.getClock());
    	DispatchZLog dispatchZLog = null;
    	if(t!=null){
    		dispatchZLog = dispatchZLogService.modify(model);
    	}else{
    		dispatchZLog = dispatchZLogService.add(model);
    	}
        return success(dispatchZLog);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("dispatchzlog:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  dispatchZLogService.delete(ids);
    	}
      return success();
    }
}
