package com.steerinfo.ems.emseotiming.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.emseotiming.model.EmsEoTiming;
import com.steerinfo.ems.emseotiming.service.IEmsEoTimingService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * EmsEoTiming RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2020-12-23 03:33
 * 类描述
 * 修订历史：
 * 日期：2020-12-23
 * 作者：generator
 * 参考：
 * 描述：EmsEoTiming RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/emseotimings")
public class EmsEoTimingController extends BaseRESTfulController {

    @Autowired
    IEmsEoTimingService emsEoTimingService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("emseotiming:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<EmsEoTiming> list = emsEoTimingService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("emseotiming:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<EmsEoTiming> list = emsEoTimingService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据EmsEoTiming对象创建")
    @ApiImplicitParam(name = "emsEoTiming", value = "详细实体emsEoTiming", required = true, dataType = "EmsEoTiming")
    //@RequiresPermissions("emseotiming:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute EmsEoTiming model){
        EmsEoTiming emsEoTiming = emsEoTimingService.add(model);
        return success(emsEoTiming);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("emseotiming:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        EmsEoTiming emsEoTiming = emsEoTimingService.getById(id);
        return success(emsEoTiming);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的emsEoTiming信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "emsEoTiming", value = "详细实体emsEoTiming", required = true, dataType = "EmsEoTiming")
    })
    //@RequiresPermissions("emseotiming:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody EmsEoTiming model){
        model.setId(id);
        EmsEoTiming emsEoTiming = emsEoTimingService.modify(model);
        return success(emsEoTiming);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("emseotiming:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  emsEoTimingService.delete(ids);
    	}
      return success();
    }
    
    @ApiOperation(value="获取数据", notes="获取数据")
    //@RequiresPermissions("emseotiming:view")
    @GetMapping(value = "/testRead")
    public RESTfulResult testRead(){
    	emsEoTimingService.getLaterData();
    	return success(111);
    	
    }
    
    @ApiOperation(value="获取数据1", notes="获取数据1")
    //@RequiresPermissions("emseotiming:view")
    @GetMapping(value = "/testData")
    public RESTfulResult testData(){
    	emsEoTimingService.getData();
    	return success("ok");
    }
}
