package com.steerinfo.ems.tcm0311.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.tcm0311.model.TCm0311;
import com.steerinfo.ems.tcm0311.service.ITCm0311Service;
import com.steerinfo.ems.tcm0312.service.ITCm0312Service;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * TCm0311 RESTful接口：
 * @author 王金涛
 * @version 1.0-SNAPSHORT 2019-10-15 09:49
 * 类描述
 * 修订历史：
 * 日期：2019-10-15
 * 作者：王金涛
 * 参考：
 * 描述：TCm0311 RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcm0311s")
public class TCm0311Controller extends BaseRESTfulController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TCm0311Controller.class);

    @Autowired
    ITCm0311Service tCm0311Service;
    @Autowired
    ITCm0312Service tCm0312Service;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0311:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0311> list = tCm0311Service.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="创建", notes="根据TCm0311对象创建")
    @ApiImplicitParam(name = "tCm0311", value = "详细实体tCm0311", required = true, dataType = "TCm0311")
    //@RequiresPermissions("tcm0311:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TCm0311 model){
        if(tCm0311Service.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        else{
            model.setXh(tCm0311Service.getMaxXh() + 1);
            model.setCreateman(JwtUtil.getUseridByToken());
            TCm0311 tCm0311 = tCm0311Service.add(model);
            return success(tCm0311);
        }
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0311:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCm0311 tCm0311 = tCm0311Service.getById(id);
        return success(tCm0311);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tCm0311信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tCm0311", value = "详细实体tCm0311", required = true, dataType = "TCm0311")
    })
    //@RequiresPermissions("tcm0311:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TCm0311 model){
        LOGGER.info("水质检测项目维护表更新前数据是" + tCm0311Service.getById(model.getItemcode()).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TCm0311 tCm0311 = tCm0311Service.modify(model);
        LOGGER.info("水质检测项目维护表更新后数据是" + tCm0311.toString());
        return success(tCm0311);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0311:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        String[] array = id.split(",");
        int failnum = 0;
        String failmsg = "";
        for (int i = 0; i < array.length; i++) {
            if(tCm0312Service.queryByProperty("sitem", array[i]).size() != 0){
                LOGGER.error("水质监测项目维护在删除第" + (i + 1) + "条数据时出现问题，id是" + array[i] + "原因是在T_CM_0312表存在关系。");
                failnum++;
                failmsg += array[i] +"在水质监测计划制定维护界面存在关系，请先删除对应关系！";
                continue;
            }
            if(tCm0311Service.getById(array[i]) == null){
                LOGGER.error("水质监测项目维护在删除第" + (i + 1) + "条数据时出现问题，水质监测项目维护不存在该" + array[i]);
                failnum++;
                failmsg += System.getProperty("line.separator") +"水质监测项目维护不存在该" + array[i];
                continue;
            }
            LOGGER.info("水质监测项目维护要删除的第" + (i + 1) + "条数据是" + tCm0311Service.getById(array[i]).toString());
            tCm0311Service.delete(array[i]);
        }
        if(failnum > 0){
            LOGGER.info(JwtUtil.getUseridByToken() + "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败的数据id:" + failmsg);
            return failed(null, "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败原因：" + failmsg);
        }
        return success();
    }

    @ApiOperation(value="未选定的监测项目多选框", notes="查询Unsitem的id和name")
    @GetMapping(value = "/getunsitemidandname/")
    public RESTfulResult getUnsitemNameAndId(@RequestParam HashMap param){
        List<TCm0311> tCm0311 = tCm0311Service.getUnsitemNameAndId(param);
        return success(tCm0311);
    }

    @ApiOperation(value="监测项目下拉框", notes="查询id和name")
    @GetMapping(value = "/getidandname/")
    public RESTfulResult getNameAndId(){
        List<TCm0311> tCm0311 = tCm0311Service.getNameAndId();
        return success(tCm0311);
    }

    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tCm0311数组信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0311", value = "详细实体tCm0311", required = true, dataType = "TCm0311")
    //@RequiresPermissions("tcm0311:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TCm0311[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("水质监测项目维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("水质监测项目维护修改前数据是" + tCm0311Service.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TCm0311 tCm0311 = tCm0311Service.modify(model[i]);
                LOGGER.info("水质监测项目表修改后数据是" + tCm0311.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "水质监测项目维护在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }


/**
    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0311:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0311> list = tCm0311Service.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
*/
}
