package com.steerinfo.ems.tcm0327.controller;

import com.steerinfo.ems.Utils.DateUtils;
import com.steerinfo.ems.tcm0327.model.TCm0327;
import com.steerinfo.ems.tcm0327.service.ITCm0327Service;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;

/**
 * TCm0327 RESTful接口：
 * 作者：王金涛
 * @version 1.0-SNAPSHORT 2019-10-29
 * 类描述
 * 修订历史：
 * 日期：2019-10-29
 * 作者：王金涛
 * 参考：
 * 描述：TCm0327 RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcm0327s")
public class TCm0327Controller extends BaseRESTfulController {

    @Autowired
    ITCm0327Service tCm0327Service;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0327:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0327> list = tCm0327Service.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0327:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tCm0327Service.delete(id);
        } else {
            tCm0327Service.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }

    @ApiOperation(value="暂存", notes="根据url的id来指定更新对象，并根据传过来的tCm0327信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0327", value = "详细实体tCm0327", required = true, dataType = "TCm0327")
    //@RequiresPermissions("tcm0327:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody TCm0327 model){
        if(model.getChangeno() == null || "".equals(model.getChangeno()) || tCm0327Service.getById(model.getChangeno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTimetoDate("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("create");
            int maxid = tCm0327Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0327Service.getMaxId()) + 1;
            model.setChangeno("GWBGD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0327Service.add(model);
            return success(model);
        }else {
            TCm0327 tCm0327 = tCm0327Service.modify(model);
            return success(tCm0327);
        }
    }

    @ApiOperation(value="提交", notes="根据url的id来指定更新对象，并根据传过来的tCm0327信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0327", value = "详细实体tCm0327", required = true, dataType = "TCm0327")
    //@RequiresPermissions("tcm0327:update")
    @PutMapping(value = "/submit/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult submit(@RequestBody TCm0327 model){
        if(model.getChangeno() == null || "".equals(model.getChangeno()) || tCm0327Service.getById(model.getChangeno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTimetoDate("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("created");
            int maxid = tCm0327Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0327Service.getMaxId()) + 1;
            model.setChangeno("GWBGD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0327Service.add(model);
            return success(model);
        }else {
            if (model.getChangetime() == null || "".equals(model.getChangetime())) {
                model.setChangetime(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("created");
            TCm0327 tCm0327 = tCm0327Service.modify(model);
            return success(tCm0327);
        }
    }

    /**
    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0327:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0327> list = tCm0327Service.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0327:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCm0327 tCm0327 = tCm0327Service.getById(id);
        return success(tCm0327);
    }

     @ApiOperation(value="创建", notes="根据TCm0327对象创建")
     @ApiImplicitParam(name = "tCm0327", value = "详细实体tCm0327", required = true, dataType = "TCm0327")
     //@RequiresPermissions("tcm0327:create")
     @PostMapping(value = "/")
     public RESTfulResult add(@ModelAttribute TCm0327 model) {
     try {
     model.setBillstatus("create");
     if(model.getCreattime() == null || "".equals(model.getCreattime())){
     model.setCreattime(new SimpleDateFormat("yyyy-MM-dd").parse(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss")));
     }
     TCm0327 tCm0327 = tCm0327Service.add(model);
     return success(tCm0327);
     }
     catch (ParseException e){
     e.printStackTrace();
     return failed();
     }
     }
    */
}
