package com.steerinfo.ems.tcm0334.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.Utils.DateUtils;
import com.steerinfo.ems.tcm0312.service.ITCm0312Service;
import com.steerinfo.ems.tcm0334.model.TCm0334;
import com.steerinfo.ems.tcm0334.service.ITCm0334Service;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * TCm0334 RESTful接口：
 * @author 王金涛
 * @version 1.0-SNAPSHORT 2019-10-15 09:49
 * 类描述
 * 修订历史：
 * 日期：2019-10-15
 * 作者：王金涛
 * 参考：
 * 描述：TCm0334 RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcm0334s")
public class TCm0334Controller extends BaseRESTfulController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TCm0334Controller.class);

    @Autowired
    ITCm0334Service tCm0334Service;
    @Autowired
    ITCm0312Service tCm0312Service;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0334:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0334> list = tCm0334Service.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="创建", notes="根据TCm0334对象创建")
    @ApiImplicitParam(name = "tCm0334", value = "详细实体tCm0334", required = true, dataType = "TCm0334")
    //@RequiresPermissions("tcm0334:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TCm0334 model){
        if(tCm0334Service.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        else {
            if(model.getCreatyear() == null || "".equals(model.getCreatyear())){
                model.setCreatyear(DateUtils.getCurrentTime("yyyy"));
            }
            model.setCreateman(JwtUtil.getUseridByToken());
            TCm0334 tCm0334 = tCm0334Service.add(model);
            return success(tCm0334);
        }
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tCm0334信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tCm0334", value = "详细实体tCm0334", required = true, dataType = "TCm0334")
    })
    //@RequiresPermissions("tcm0334:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TCm0334 model){
        LOGGER.info("水质监测点维护修改前的数据是" + tCm0334Service.getById(id).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TCm0334 tCm0334 = tCm0334Service.modify(model);
        LOGGER.info("水质监测点维护修改后的数据是" + tCm0334.toString());
        return success(tCm0334);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0334:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        String[] array = id.split(",");
        int failnum = 0;
        String failmsg = "";
        for (int i = 0; i < array.length; i++) {
            if(tCm0312Service.queryByProperty("powermonitorname", array[i]).size() != 0){
                LOGGER.error("水质监测点维护在删除第" + (i + 1) + "条数据时出现问题，id是" + array[i] + "原因是在T_CM_0312表存在关系。");
                failnum++;
                failmsg += array[i] +"在水质监测计划制定维护界面存在关系，请先删除对应关系！";
                continue;
            }
            if(tCm0334Service.getById(array[i]) == null){
                LOGGER.error("水质监测点维护在删除第" + (i + 1) + "条数据时出现问题，水质监测点维护不存在该" + array[i]);
                failnum++;
                failmsg += "水质监测点维护不存在该" + array[i];
                continue;
            }
            LOGGER.info("水质监测点维护要删除的第" + (i + 1) + "条数据是" + tCm0334Service.getById(array[i]).toString());
            tCm0334Service.delete(array[i]);
        }
        if(failnum > 0){
            LOGGER.info(JwtUtil.getUseridByToken() + "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败的数据id:" + failmsg);
            return failed(null, "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败原因：" + failmsg);
        }
        return success();
    }

    @ApiOperation(value="监测点下拉框", notes="查询id和name")
    @GetMapping(value = "/getidandname/")
    public RESTfulResult getNameAndId(){
        List<TCm0334> tCm0334 = tCm0334Service.getNameAndId();
        return success(tCm0334);
    }

    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tCm0334数组信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0334", value = "详细实体tCm0334", required = true, dataType = "TCm0334")
    //@RequiresPermissions("tcm0334:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TCm0334[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("水质监测点维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("水质监测点维护修改前数据是" + tCm0334Service.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TCm0334 tCm0334 = tCm0334Service.modify(model[i]);
                LOGGER.info("水质监测点维护修改后数据是" + tCm0334.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "水质监测点维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }
/**

 @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
 @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
 //@RequiresPermissions("tcm0334:view")
 @GetMapping(value = "/{id}")
 public RESTfulResult get(@PathVariable String id){
 TCm0334 tCm0334 = tCm0334Service.getById(id);
 return success(tCm0334);
 }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0334:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0334> list = tCm0334Service.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
*/
}
