package com.steerinfo.ems.tcm0349.controller;

import com.steerinfo.ems.Utils.DateUtils;
import com.steerinfo.ems.tcm0349.model.TCm0349;
import com.steerinfo.ems.tcm0349.service.ITCm0349Service;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;

/**
 * TCm0349 RESTful接口：
 * 作者：王金涛
 * @version 1.0-SNAPSHORT 2019-10-29
 * 类描述
 * 修订历史：
 * 日期：2019-10-29
 * 作者：王金涛
 * 参考：
 * 描述：TCm0349 RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcm0349s")
public class TCm0349Controller extends BaseRESTfulController {

    @Autowired
    ITCm0349Service tCm0349Service;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0349:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0349> list = tCm0349Service.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0349:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tCm0349Service.delete(id);
        } else {
            tCm0349Service.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }

    @ApiOperation(value="暂存", notes="根据url的id来指定更新对象，并根据传过来的tCm0349信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0349", value = "详细实体tCm0349", required = true, dataType = "TCm0349")
    //@RequiresPermissions("tcm0349:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody TCm0349 model){
        if(model.getChangeno() == null || "".equals(model.getChangeno()) || tCm0349Service.getById(model.getChangeno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("create");
            int maxid = tCm0349Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0349Service.getMaxId()) + 1;
            model.setChangeno("ZGD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0349Service.add(model);
            return success(model);
        }else{
            TCm0349 tCm0349 = tCm0349Service.modify(model);
            return success(tCm0349);
        }
    }

    @ApiOperation(value="提交", notes="根据url的id来指定更新对象，并根据传过来的tCm0349信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0349", value = "详细实体tCm0349", required = true, dataType = "TCm0349")
    //@RequiresPermissions("tcm0349:update")
    @PutMapping(value = "/submit/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult submit(@RequestBody TCm0349 model){
        if(model.getChangeno() == null || "".equals(model.getChangeno()) || tCm0349Service.getById(model.getChangeno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("create");
            int maxid = tCm0349Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0349Service.getMaxId()) + 1;
            model.setChangeno("ZGD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0349Service.add(model);
            return success(model);
        }else {
            model.setBillstatus("created");
            TCm0349 tCm0349 = tCm0349Service.modify(model);
            return success(tCm0349);
        }
    }
    /**

     @ApiOperation(value="获取列表", notes="分页模糊查询")
     @ApiImplicitParams({
     @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
     @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
     })
     //@RequiresPermissions("tcm0349:view")
     @GetMapping(value = "/like/")
     public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
     PageList<TCm0349> list = tCm0349Service.queryLikeForPage(parmas, pageNum, pageSize);
     return success(list);
     }

     @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
     @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
     //@RequiresPermissions("tcm0349:view")
     @GetMapping(value = "/{id}")
     public RESTfulResult get(@PathVariable String id){
     TCm0349 tCm0349 = tCm0349Service.getById(id);
     return success(tCm0349);
     }

     @ApiOperation(value="创建", notes="根据TCm0349对象创建")
     @ApiImplicitParam(name = "tCm0349", value = "详细实体tCm0349", required = true, dataType = "TCm0349")
     //@RequiresPermissions("tcm0349:create")
     @PostMapping(value = "/")
     public RESTfulResult add(@ModelAttribute TCm0349 model){
     model.setBillstatus("create");
     if(model.getCreattime() == null || "".equals(model.getCreattime())){
     model.setCreattime(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss"));
     }
     TCm0349 tCm0349 = tCm0349Service.add(model);
     return success(tCm0349);
     }

     */
}
