package com.steerinfo.ems.tcm0350.controller;

import com.steerinfo.ems.Utils.DateUtils;
import com.steerinfo.ems.tcm0350.model.TCm0350;
import com.steerinfo.ems.tcm0350.service.ITCm0350Service;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;

/**
 * TCm0350 RESTful接口：
 * 作者：王金涛
 * @version 1.0-SNAPSHORT 2019-10-29
 * 类描述
 * 修订历史：
 * 日期：2019-10-29
 * 作者：王金涛
 * 参考：
 * 描述：TCm0350 RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcm0350s")
public class TCm0350Controller extends BaseRESTfulController {

    @Autowired
    ITCm0350Service tCm0350Service;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0350:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0350> list = tCm0350Service.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0350:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tCm0350Service.delete(id);
        } else {
            tCm0350Service.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }

    @ApiOperation(value="暂存", notes="根据url的id来指定更新对象，并根据传过来的tCm0350信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0350", value = "详细实体tCm0350", required = true, dataType = "TCm0350")
    //@RequiresPermissions("tcm0350:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody TCm0350 model){
        if(model.getPunishno() == null || "".equals(model.getPunishno()) || tCm0350Service.getById(model.getPunishno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTimetoDate("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("create");
            int maxid = tCm0350Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0350Service.getMaxId()) + 1;
            model.setPunishno("CFD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0350Service.add(model);
            return success(model);
        }else {
            TCm0350 tCm0350 = tCm0350Service.modify(model);
            return success(tCm0350);
        }
    }

    @ApiOperation(value="提交", notes="根据url的id来指定更新对象，并根据传过来的tCm0350信息来更新详细信息")
    @ApiImplicitParam(name = "tCm0350", value = "详细实体tCm0350", required = true, dataType = "TCm0350")
    //@RequiresPermissions("tcm0350:update")
    @PutMapping(value = "/submit/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult submit(@RequestBody TCm0350 model){
        if(model.getPunishno() == null || "".equals(model.getPunishno()) || tCm0350Service.getById(model.getPunishno()) == null){
            if(model.getCreattime() == null || "".equals(model.getCreattime())){
                model.setCreattime(DateUtils.getCurrentTimetoDate("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("created");
            int maxid = tCm0350Service.getMaxId() == null ? 1 : Integer.parseInt(tCm0350Service.getMaxId()) + 1;
            model.setPunishno("CFD_" + DateUtils.getCurrentTime("yyyyMMdd") + "_" + String.format("%04d", maxid));
            tCm0350Service.add(model);
            return success(model);
        }else {
            if (model.getHandletime() == null || "".equals(model.getHandletime())) {
                model.setHandletime(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss"));
            }
            model.setBillstatus("created");
            TCm0350 tCm0350 = tCm0350Service.modify(model);
            return success(tCm0350);
        }
    }

/**
    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcm0350:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCm0350 tCm0350 = tCm0350Service.getById(id);
        return success(tCm0350);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcm0350:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCm0350> list = tCm0350Service.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }

 @ApiOperation(value="创建", notes="根据TCm0327对象创建")
 @ApiImplicitParam(name = "tCm0327", value = "详细实体tCm0327", required = true, dataType = "TCm0327")
 //@RequiresPermissions("tcm0327:create")
 @PostMapping(value = "/")
 public RESTfulResult add(@ModelAttribute TCm0327 model) {
 try {
 model.setBillstatus("create");
 if(model.getCreattime() == null || "".equals(model.getCreattime())){
 model.setCreattime(new SimpleDateFormat("yyyy-MM-dd").parse(DateUtils.getCurrentTime("yyyy-MM-dd HH:mm:ss")));
 }
 TCm0327 tCm0327 = tCm0327Service.add(model);
 return success(tCm0327);
 }
 catch (ParseException e){
 e.printStackTrace();
 return failed();
 }
 }
    */
}
