package com.steerinfo.ems.tcmcodeitem.controller;

import com.steerinfo.ems.tcmcodeitem.model.TCmCodeitem;
import com.steerinfo.ems.tcmcodeitem.service.ITCmCodeitemService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * TCmCodeitem RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-28 08:03
 * 类描述
 * 修订历史：
 * 日期：2019-10-28
 * 作者：generator
 * 参考：
 * 描述：TCmCodeitem RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcmcodeitems")
public class TCmCodeitemController extends BaseRESTfulController {

    @Autowired
    ITCmCodeitemService tCmCodeitemService;

    @ApiOperation(value="获取特定的id和name")
    @ApiImplicitParam(paramType = "path", name = "value", value = "字段", required = true, dataType = "String")
    @GetMapping(value = "/{value}")
    public RESTfulResult list(@PathVariable String value){
        List<TCmCodeitem> list = tCmCodeitemService.queryByProperty("typecode", value);
        return success(list);
    }
/**
    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmcodeitem:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmCodeitem> list = tCmCodeitemService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmcodeitem:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmCodeitem> list = tCmCodeitemService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TCmCodeitem对象创建")
    @ApiImplicitParam(name = "tCmCodeitem", value = "详细实体tCmCodeitem", required = true, dataType = "TCmCodeitem")
    //@RequiresPermissions("tcmcodeitem:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TCmCodeitem model){
        TCmCodeitem tCmCodeitem = tCmCodeitemService.add(model);
        return success(tCmCodeitem);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmcodeitem:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCmCodeitem tCmCodeitem = tCmCodeitemService.getById(id);
        return success(tCmCodeitem);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tCmCodeitem信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tCmCodeitem", value = "详细实体tCmCodeitem", required = true, dataType = "TCmCodeitem")
    })
    //@RequiresPermissions("tcmcodeitem:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TCmCodeitem model){
        model.setId(id);
        TCmCodeitem tCmCodeitem = tCmCodeitemService.modify(model);
        return success(tCmCodeitem);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmcodeitem:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tCmCodeitemService.delete(id);
        } else {
            tCmCodeitemService.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }
*/
}
