package com.steerinfo.ems.tcmdatasourceexception.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.ems.tcmdatasourceexception.model.TCmDatasourceException;
import com.steerinfo.ems.tcmdatasourceexception.service.ITCmDatasourceExceptionService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

/**
 * TCmDatasourceException RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-15 04:18
 * 类描述
 * 修订历史：
 * 日期：2019-10-15
 * 作者：generator
 * 参考：
 * 描述：TCmDatasourceException RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcmdatasourceexceptions")
public class TCmDatasourceExceptionController extends BaseRESTfulController {

    @Autowired
    ITCmDatasourceExceptionService tCmDatasourceExceptionService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmdatasourceexception:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmDatasourceException> list = tCmDatasourceExceptionService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmdatasourceexception:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmDatasourceException> list = tCmDatasourceExceptionService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TCmDatasourceException对象创建")
    @ApiImplicitParam(name = "tCmDatasourceException", value = "详细实体tCmDatasourceException", required = true, dataType = "TCmDatasourceException")
    //@RequiresPermissions("tcmdatasourceexception:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@RequestBody TCmDatasourceException model){
        String str = model.getDatasourcename() +model.getTimegranid() +model.getItemid();
        Set<String> strings = tCmDatasourceExceptionService.uniqueKey();
        for (String string : strings) {
            if(string.equals(str)){
                return  failed(model,"以有相同数据,请勿重复添加");
            }
        }
        TCmDatasourceException tCmDatasourceException = tCmDatasourceExceptionService.add(model);
        return success(tCmDatasourceException);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmdatasourceexception:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCmDatasourceException tCmDatasourceException = tCmDatasourceExceptionService.getById(id);
        return success(tCmDatasourceException);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tCmDatasourceException信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tCmDatasourceException", value = "详细实体tCmDatasourceException", required = true, dataType = "TCmDatasourceException")
    })
    //@RequiresPermissions("tcmdatasourceexception:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TCmDatasourceException model){
        model.setId(id);
        TCmDatasourceException tCmDatasourceException = tCmDatasourceExceptionService.modify(model);
        return success(tCmDatasourceException);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmdatasourceexception:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tCmDatasourceExceptionService.delete(id);
        } else {
            tCmDatasourceExceptionService.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }

    @ApiOperation(value ="获取详细信息",notes = "根据url传过来得字符逗号分割进行查询")
    @ApiImplicitParam(paramType = "path", name = "str", required = true, dataType = "String")
    @GetMapping(value = "/getOne/{str}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult getOne(@PathVariable String str){
        String[] split = str.split(",");
        String datasourcename = split[0];
        String timegranid = split[1];
        TCmDatasourceException tc = new TCmDatasourceException();
        tc.setDatasourcename(datasourcename);
        tc.setTimegranid(timegranid);
        TCmDatasourceException one = tCmDatasourceExceptionService.getOne(tc);
        return  success(one);
    }

}
