package com.steerinfo.ems.tcmtimedefinition.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.tcmtimedefinition.model.TCmTimeDefinition;
import com.steerinfo.ems.tcmtimedefinition.service.ITCmTimeDefinitionService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * TCmTimeDefinition RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-11
 * 类描述
 * 修订历史：
 * 日期：2019-10-11
 * 作者：generator
 * 参考：
 * 描述：TCmTimeDefinition RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tcmtimedefinitions")
public class TCmTimeDefinitionController extends BaseRESTfulController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TCmTimeDefinitionController.class);

    @Autowired
    ITCmTimeDefinitionService tCmTimeDefinitionService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmtimedefinition:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmTimeDefinition> list = tCmTimeDefinitionService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("tcmtimedefinition:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TCmTimeDefinition> list = tCmTimeDefinitionService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TCmTimeDefinition对象创建")
    @ApiImplicitParam(name = "tCmTimeDefinition", value = "详细实体tCmTimeDefinition", required = true, dataType = "TCmTimeDefinition")
    //@RequiresPermissions("tcmtimedefinition:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TCmTimeDefinition model){
        if(tCmTimeDefinitionService.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        else{
            model.setCreateman(JwtUtil.getUseridByToken());
            TCmTimeDefinition tCmTimeDefinition = tCmTimeDefinitionService.add(model);
            return success(tCmTimeDefinition);
        }
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmtimedefinition:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TCmTimeDefinition tCmTimeDefinition = tCmTimeDefinitionService.getById(id);
        return success(tCmTimeDefinition);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tCmTimeDefinition信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tCmTimeDefinition", value = "详细实体tCmTimeDefinition", required = true, dataType = "TCmTimeDefinition")
    })
    //@RequiresPermissions("tcmtimedefinition:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TCmTimeDefinition model){
        LOGGER.info("时间序列表修改前数据是" + tCmTimeDefinitionService.getById(id).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TCmTimeDefinition tCmTimeDefinition = tCmTimeDefinitionService.modify(model);
        LOGGER.info("时间序列表修改后数据是" + tCmTimeDefinition.toString());
        return success(tCmTimeDefinition);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("tcmtimedefinition:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        String[] array = id.split(",");
        if(array.length >= 1) {
            int failnum = 0;
            String failmsg = "";
            for (int i = 0; i < array.length; i++) {
                try {
                    LOGGER.info("时间序列表要删除的第" + (i + 1) + "条数据是" + tCmTimeDefinitionService.getById(array[i]).toString());
                    tCmTimeDefinitionService.delete(array[i]);
                }
                catch(Exception e){
                    failnum++;
                    failmsg += "时间序列维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + array[i];
                }
            }
            if(failnum > 0){
                LOGGER.info(JwtUtil.getUseridByToken() + "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
                return success(null, "51","本次修改失败" + failnum + "条,失败id:" + failmsg);
            }
            return success();
        }
        else {
            return failed(null, "请输入要删除的id");
        }
    }

    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tCmTimeDefinition数组信息来更新详细信息")
    @ApiImplicitParam(name = "tCmTimeDefinition", value = "详细实体tCmTimeDefinition", required = true, dataType = "TCmTimeDefinition")
    //@RequiresPermissions("tcmtimedefinition:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TCmTimeDefinition[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("时间序列维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("时间序列维护修改前数据是" + tCmTimeDefinitionService.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TCmTimeDefinition tCmTimeDefinition = tCmTimeDefinitionService.modify(model[i]);
                LOGGER.info("时间序列维护修改后数据是" + tCmTimeDefinition.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "时间序列维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }
}
