package com.steerinfo.ems.trmenergyconvert.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.ems.trmenergyconvert.model.TRmEnergyConvert;
import com.steerinfo.ems.trmenergyconvert.service.ITRmEnergyConvertService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * TRmEnergyConvert RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-17 03:32
 * 类描述
 * 修订历史：
 * 日期：2019-10-17
 * 作者：generator
 * 参考：
 * 描述：TRmEnergyConvert RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmenergyconverts")
public class TRmEnergyConvertController extends BaseRESTfulController {

    @Autowired
    ITRmEnergyConvertService tRmEnergyConvertService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergyconvert:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyConvert> list = tRmEnergyConvertService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergyconvert:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyConvert> list = tRmEnergyConvertService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmEnergyConvert对象创建")
    @ApiImplicitParam(name = "tRmEnergyConvert", value = "详细实体tRmEnergyConvert", required = true, dataType = "TRmEnergyConvert")
    //@RequiresPermissions("trmenergyconvert:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmEnergyConvert model){
        TRmEnergyConvert tRmEnergyConvert = tRmEnergyConvertService.add(model);
        return success(tRmEnergyConvert);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergyconvert:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmEnergyConvert tRmEnergyConvert = tRmEnergyConvertService.getById(id);
        return success(tRmEnergyConvert);
    }

    @ApiOperation(value="更新详细信息", notes="根据主键(energyid, clock)来指定更新对象，并根据传过来的tRmEnergyConvert信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "tRmEnergyConvert", value = "详细实体tRmEnergyConvert", required = true, dataType = "TRmEnergyConvert")
    })
    //@RequiresPermissions("trmenergyconvert:update")
    @PutMapping(value = "/updateForUKey/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult updateForUKey(@RequestBody TRmEnergyConvert model){
    	if (model == null) {
    		return failed(null, "请传入主键参数");
    	}
    	if (model.getEnergyid()== null || model.getEnergyid().isEmpty()) {
    		return failed(null, "请传入主键参数energyid");
    	}
    	if (model.getClock() == null || model.getClock().isEmpty()) {
    		return failed(null, "请传入主键参数clock");
    	}
    	TRmEnergyConvert tRmEnergyConvert = tRmEnergyConvertService.updateForUKey(model);
        return success(tRmEnergyConvert);
    }

    @ApiOperation(value="删除", notes="根据主键(energyid, clock)删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergyconvert:delete")
    @DeleteMapping(value = "/del/")
    public RESTfulResult delete(@RequestParam HashMap<String, Object> parmas){
    	if (parmas == null || parmas.size() < 1) {
    		return failed(null, "请传入主键参数");
    	}
    	if (parmas.get("energyid") == null || parmas.get("energyid").toString().isEmpty()) {
    		return failed(null, "请传入主键参数energyid");
    	}
    	if (parmas.get("clock") == null || parmas.get("clock").toString().isEmpty()) {
    		return failed(null, "请传入主键参数clock");
    	}
    	tRmEnergyConvertService.deleteByUPrimaryKey(parmas);
        return success();
    }
}
