package com.steerinfo.ems.trmenergyprice.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.trmenergyprice.model.TRmEnergyPrice;
import com.steerinfo.ems.trmenergyprice.service.ITRmEnergyPriceService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.xml.crypto.Data;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * TRmEnergyPrice RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2020-09-17 02:53
 * 类描述
 * 修订历史：
 * 日期：2020-09-17
 * 作者：generator
 * 参考：
 * 描述：TRmEnergyPrice RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmenergyprices")
public class TRmEnergyPriceController extends BaseRESTfulController {

    @Autowired
    ITRmEnergyPriceService tRmEnergyPriceService;
    
    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergyprice:view")
    @GetMapping(value = "/getData")
    public RESTfulResult getData(@RequestParam HashMap<String,Object> parmas,Integer pageNum, Integer pageSize){
        PageList<Map<String, Object>> list = tRmEnergyPriceService.getDataForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergyprice:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyPrice> list = tRmEnergyPriceService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergyprice:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyPrice> list = tRmEnergyPriceService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmEnergyPrice对象创建")
    @ApiImplicitParam(name = "tRmEnergyPrice", value = "详细实体tRmEnergyPrice", required = true, dataType = "TRmEnergyPrice")
    //@RequiresPermissions("trmenergyprice:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmEnergyPrice model){
    	HashMap<String,Object> parmas = new HashMap<String,Object>();
		parmas.put("energyid", model.getEnergyid());
		if(tRmEnergyPriceService.query(parmas).size()>0){
			return failed(null, "该能介类型价格信息已存在");
		}
    	model.setCreateman(JwtUtil.getUseridByToken());
    	model.setCreatetime(new Date());
        TRmEnergyPrice tRmEnergyPrice = tRmEnergyPriceService.add(model);
        return success(tRmEnergyPrice);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergyprice:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmEnergyPrice tRmEnergyPrice = tRmEnergyPriceService.getById(id);
        return success(tRmEnergyPrice);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmEnergyPrice信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmEnergyPrice", value = "详细实体tRmEnergyPrice", required = true, dataType = "TRmEnergyPrice")
    })
    //@RequiresPermissions("trmenergyprice:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody TRmEnergyPrice[] model) {
        for (int i = 0; i < model.length; i++) {
        	if(model[i].getId()==null){
        		model[i].setCreateman(JwtUtil.getUseridByToken());
        		model[i].setCreatetime(new Date());
        		tRmEnergyPriceService.add(model[i]);
        	}else{
        		model[i].setUpdateman(JwtUtil.getUseridByToken());
                model[i].setUpdatetime(new Date());
                tRmEnergyPriceService.modify(model[i]);
        	}
        }
        return success();
    }
    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergyprice:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  tRmEnergyPriceService.delete(ids);
    	}

      return success();
    }
}
