package com.steerinfo.ems.trmenergytype.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.trmcalpoint.service.ITRmCalpointService;
import com.steerinfo.ems.trmenergy.service.ITRmEnergyService;
import com.steerinfo.ems.trmenergytype.model.TRmEnergyType;
import com.steerinfo.ems.trmenergytype.service.ITRmEnergyTypeService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * TRmEnergyType RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-09-09 06:40
 * 类描述
 * 修订历史：
 * 日期：2019-09-09
 * 作者：generator
 * 参考：
 * 描述：TRmEnergyType RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmenergytypes")
public class TRmEnergyTypeController extends BaseRESTfulController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TRmEnergyTypeController.class);

    @Autowired
    ITRmEnergyTypeService tRmEnergyTypeService;
    @Autowired
    ITRmEnergyService tRmEnergyService;
    @Autowired
    ITRmCalpointService tRmCalpointService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergytype:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyType> list = tRmEnergyTypeService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmenergytype:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmEnergyType> list = tRmEnergyTypeService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmEnergyType对象创建")
    @ApiImplicitParam(name = "tRmEnergyType", value = "详细实体tRmEnergyType", required = true, dataType = "TRmEnergyType")
    //@RequiresPermissions("trmenergytype:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmEnergyType model){
        if(tRmEnergyTypeService.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        else{
            model.setCreateman(JwtUtil.getUseridByToken());
            TRmEnergyType tRmEnergyType = tRmEnergyTypeService.add(model);
            return success(tRmEnergyType);
        }
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergytype:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmEnergyType tRmEnergyType = tRmEnergyTypeService.getById(id);
        return success(tRmEnergyType);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmEnergyType信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmEnergyType", value = "详细实体tRmEnergyType", required = true, dataType = "TRmEnergyType")
    })
    //@RequiresPermissions("trmenergytype:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TRmEnergyType model){
        LOGGER.info("能介类型维护表修改前数据是" + tRmEnergyTypeService.getById(id).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TRmEnergyType tRmEnergyType = tRmEnergyTypeService.modify(model);
        LOGGER.info("能介类型维护表修改后数据是" + tRmEnergyType.toString());
        return success(tRmEnergyType);
    }

    @ApiOperation(value="删除", notes="根据用户ID来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmenergytype:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        int failnum = 0;
        String failmsg = "";
        List<String> Ids = Arrays.asList(id.split(","));
        for(int i = 0;i < Ids.size();i++) {
            if(tRmEnergyService.queryByProperty("energytypeid", Ids.get(i)).size() != 0){
                failnum++;
                failmsg += Ids.get(i) + "在能介信息维护界面存在关系，请先删除对应关系！";
                continue;
            }
            if(tRmCalpointService.queryByProperty("energytypeid", Ids.get(i)).size() != 0){
                failnum++;
                failmsg += Ids.get(i) + "在计量点信息维护界面存在关系，请先删除对应关系！";
                continue;
            }
            LOGGER.info("能介类型维护表要删除的第" + (i + 1) + "条数据是" + tRmEnergyTypeService.getById(Ids.get(i)).toString());
            tRmEnergyTypeService.delete(Ids.get(i));
        }
        if(failnum > 0){
            return failed(null, "本次共删除" + Ids.size() + "条数据,其中成功" + (Ids.size() - failnum) + "条,失败" + failnum + "条,失败原因：" + failmsg);
        }
        return success();
    }

    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tRmEnergyType数组信息来更新详细信息")
    @ApiImplicitParam(name = "tRmEnergyType", value = "详细实体tRmEnergyType", required = true, dataType = "TRmEnergyType")
    //@RequiresPermissions("trmworkproc:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TRmEnergyType[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("能介类型维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("能介类型维护修改前数据是" + tRmEnergyTypeService.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TRmEnergyType tRmEnergyType = tRmEnergyTypeService.modify(model[i]);
                LOGGER.info("能介类型维护修改后数据是" + tRmEnergyType.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "能介类型维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }

    @ApiOperation(value="能介类型下拉框", notes="查询id and name")
    @GetMapping(value = "/selectNameAndId/")
    public RESTfulResult selectNameAndId(){
        Set<TRmEnergyType> tRmEnergyTypes = tRmEnergyTypeService.selectNameAndId();
        return  success(tRmEnergyTypes);
    }

/**
 @ApiOperation(value="获取详细信息", notes="根据详细实体tRmEnergyType来获取详细信息")
 @ApiImplicitParam(name = "tRmEnergyType", value = "详细实体tRmEnergyType", required = true, dataType = "TRmEnergyType")
 //@RequiresPermissions("trmenergytype:view")
 @PostMapping(value = "/get/", produces  = "application/json;charset=UTF-8")
 public RESTfulResult get(@RequestBody TRmEnergyType model){
 Map<String, Object> parameters = new HashMap();
 parameters.put("id",model.getId());
 parameters.put("name",model.getName());
 TRmEnergyType tRmEnergyType = tRmEnergyTypeService.getByParams(parameters);
 return success(tRmEnergyType);
 }
 */
}
