package com.steerinfo.ems.trmindex.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.ems.trmindex.model.TRmIndex;
import com.steerinfo.ems.trmindex.service.ITRmIndexService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;

/**
 * TRmIndex RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-19 11:17
 * 类描述
 * 修订历史：
 * 日期：2019-10-19
 * 作者：generator
 * 参考：
 * 描述：TRmIndex RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmindexs")
public class TRmIndexController extends BaseRESTfulController {

    @Autowired
    ITRmIndexService tRmIndexService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmindex:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmIndex> list = tRmIndexService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmindex:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmIndex> list = tRmIndexService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmIndex对象创建")
    @ApiImplicitParam(name = "tRmIndex", value = "详细实体tRmIndex", required = true, dataType = "TRmIndex")
    //@RequiresPermissions("trmindex:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmIndex model){
        TRmIndex tRmIndex = tRmIndexService.add(model);
        return success(tRmIndex);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmindex:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmIndex tRmIndex = tRmIndexService.getById(id);
        return success(tRmIndex);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmIndex信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmIndex", value = "详细实体tRmIndex", required = true, dataType = "TRmIndex")
    })
    //@RequiresPermissions("trmindex:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TRmIndex model){
        model.setId(id);
        TRmIndex tRmIndex = tRmIndexService.modify(model);
        return success(tRmIndex);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmindex:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tRmIndexService.delete(id);
        } else {
            tRmIndexService.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }


}
