package com.steerinfo.ems.trmindextype.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.ems.trmindexitem.mapper.TRmIndexItemMapper;
import com.steerinfo.ems.trmindexitem.model.TRmIndexItem;
import com.steerinfo.ems.trmindextype.model.TRmIndexType;
import com.steerinfo.ems.trmindextype.service.ITRmIndexTypeService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

/**
 * TRmIndexType RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-10-22 04:44
 * 类描述
 * 修订历史：
 * 日期：2019-10-22
 * 作者：generator
 * 参考：
 * 描述：TRmIndexType RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmindextypes")
public class TRmIndexTypeController extends BaseRESTfulController {

    @Autowired
    ITRmIndexTypeService tRmIndexTypeService;
    
    @Autowired
    private TRmIndexItemMapper tRmIndexItemMapper;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmindextype:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmIndexType> list = tRmIndexTypeService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmindextype:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmIndexType> list = tRmIndexTypeService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmIndexType对象创建")
    @ApiImplicitParam(name = "tRmIndexType", value = "详细实体tRmIndexType", required = true, dataType = "TRmIndexType")
    //@RequiresPermissions("trmindextype:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@RequestBody TRmIndexType model){
        String str = model.getIndexcol() +model.getIndexid();
        Set<String> combinedPrimaryKeyS = tRmIndexTypeService.getCombinedPrimaryKey();
        for (String combinedPrimaryKey : combinedPrimaryKeyS) {
            if(str.equals(combinedPrimaryKey)){
                return failed(500,"指标列和指标类型代码已经重复,无法添加");
            }
        }
        TRmIndexType tRmIndexType = tRmIndexTypeService.add(model);
        return success(tRmIndexType);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmindextype:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmIndexType tRmIndexType = tRmIndexTypeService.getById(id);
        return success(tRmIndexType);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmIndexType信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmIndexType", value = "详细实体tRmIndexType", required = true, dataType = "TRmIndexType")
    })
    //@RequiresPermissions("trmindextype:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@RequestBody TRmIndexType model){
        TRmIndexType tRmIndexType = tRmIndexTypeService.modify(model);
        return success(tRmIndexType);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmindextype:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        String[] split = id.split(",");
        String indexcol =  split[0];
        String indexid = split[1];
        
        HashMap<String, Object> parmas2 = new HashMap<String, Object>();
    	parmas2.put("indextype", indexid);
    	List<TRmIndexItem> lm = tRmIndexItemMapper.selectByParameters(parmas2);
    	if(lm!=null && lm.size()>0){
			return failed(null, "[" + indexid + "]下有子项，不能删除");
		}
    	
        TRmIndexType tRmIndexType = new TRmIndexType();
        tRmIndexType.setIndexcol(indexcol);
        tRmIndexType.setIndexid(indexid);
        tRmIndexTypeService.deleteByCompositePrimaryKey(tRmIndexType);
        return success();
    }

   @ApiOperation(value = "查询",notes = "根据类型来查询")
    @ApiImplicitParam(paramType = "path",name = "indexcol",value = "indexcol",required = true,dataType = "String")
    @GetMapping("/getIdAndName")
    public RESTfulResult getIdAndName(String indexcol){
       List<TRmIndexType> idAndName = tRmIndexTypeService.getIdAndName(indexcol);
       return success(idAndName);
   }
}
