package com.steerinfo.ems.trml1source.controller;

import com.steerinfo.ems.trml1source.model.TRmL1source;
import com.steerinfo.ems.trml1source.service.ITRmL1sourceService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * TRmL1source RESTful接口：
 * @author 王金涛
 * @version 1.0-SNAPSHORT 2019-10-12
 * 类描述
 * 修订历史：
 * 日期：2019-10-12
 * 作者：王金涛
 * 参考：
 * 描述：TRmL1source RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trml1sources")
public class TRmL1sourceController extends BaseRESTfulController {

    @Autowired
    ITRmL1sourceService tRmL1sourceService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trml1source:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmL1source> list = tRmL1sourceService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trml1source:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmL1source> list = tRmL1sourceService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmL1source对象创建")
    @ApiImplicitParam(name = "tRmL1source", value = "详细实体tRmL1source", required = true, dataType = "TRmL1source")
    //@RequiresPermissions("trml1source:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmL1source model){
        if(tRmL1sourceService.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        else{
            TRmL1source tRmL1source = tRmL1sourceService.add(model);
            return success(tRmL1source);
        }
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trml1source:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmL1source tRmL1source = tRmL1sourceService.getById(id);
        return success(tRmL1source);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmL1source信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmL1source", value = "详细实体tRmL1source", required = true, dataType = "TRmL1source")
    })
    //@RequiresPermissions("trml1source:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TRmL1source model){
        model.setId(id);
        TRmL1source tRmL1source = tRmL1sourceService.modify(model);
        return success(tRmL1source);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trml1source:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        if (StringUtils.indexOf(id, ",") == -1) {
            tRmL1sourceService.delete(id);
        } else {
            tRmL1sourceService.delete(Arrays.asList(id.split(",")));
        }
        return success();
    }
    @ApiOperation(value="计量点数据来源下拉框", notes="查询id和name")
    @GetMapping(value = "/getidandname/")
    public RESTfulResult getNameAndId(){
        List<TRmL1source> tRmL1source = tRmL1sourceService.getNameAndId();
        return success(tRmL1source);
    }
}
