package com.steerinfo.ems.trmlocation.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.trmcalpoint.service.ITRmCalpointService;
import com.steerinfo.ems.trmlocation.model.TRmLocation;
import com.steerinfo.ems.trmlocation.service.ITRmLocationService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * TRmLocation RESTful接口：
 * @author 王金涛
 * @version 1.0-SNAPSHORT 2019-10-11
 * 类描述
 * 修订历史：
 * 日期：2019-10-11
 * 作者：王金涛
 * 参考：
 * 描述：TRmLocation RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmlocations")
public class TRmLocationController extends BaseRESTfulController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TRmLocationController.class);

    @Autowired
    ITRmLocationService tRmLocationService;
    @Autowired
    ITRmCalpointService tRmCalpointService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmlocation:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmLocation> list = tRmLocationService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmlocation:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmLocation> list = tRmLocationService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmLocation对象创建")
    @ApiImplicitParam(name = "tRmLocation", value = "详细实体tRmLocation", required = true, dataType = "TRmLocation")
    //@RequiresPermissions("trmlocation:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmLocation model){
        if (model.getId() == null || "".equals(model.getId())){
            int maxid = tRmLocationService.getMaxId() == null ? 1 : Integer.parseInt(tRmLocationService.getMaxId()) + 1;
            model.setId("DL" + String.format("%02d", maxid));
        }
        if(tRmLocationService.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        }
        model.setCreateman(JwtUtil.getUseridByToken());
        TRmLocation tRmLocation = tRmLocationService.add(model);
        return success(tRmLocation);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmlocation:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmLocation tRmLocation = tRmLocationService.getById(id);
        return success(tRmLocation);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmLocation信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmLocation", value = "详细实体tRmLocation", required = true, dataType = "TRmLocation")
    })
    //@RequiresPermissions("trmlocation:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TRmLocation model){
        LOGGER.info("位置单元表修改前数据是" + tRmLocationService.getById(id).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TRmLocation tRmLocation = tRmLocationService.modify(model);
        LOGGER.info("位置单元表修改后数据是" + tRmLocation.toString());
        return success(tRmLocation);
    }

    @ApiOperation(value="删除", notes="根据用户ID来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmlocation:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        int failnum = 0;
        String failmsg = "";
        List<String> Ids = Arrays.asList(id.split(","));
        for(int i = 0;i < Ids.size();i++) {
            if(tRmCalpointService.queryByProperty("locateid", Ids.get(i)).size() != 0){
                failnum++;
                failmsg += Ids.get(i) + "在计量点信息维护界面存在关系，请先删除对应关系！";
                continue;
            }
            LOGGER.info("位置单元表要删除的第" + (i + 1) + "条数据是" + tRmLocationService.getById(Ids.get(i)).toString());
            tRmLocationService.delete(Ids.get(i));
        }
        if(failnum > 0){
            return failed(null, "本次共删除" + Ids.size() + "条数据,其中成功" + (Ids.size() - failnum) + "条,失败" + failnum + "条,失败原因：" + failmsg);
        }
        return success();
    }
    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tRmWorkproc数组信息来更新详细信息")
    @ApiImplicitParam(name = "tRmLocation", value = "详细实体tRmLocation", required = true, dataType = "TRmLocation")
    //@RequiresPermissions("trmworkproc:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TRmLocation[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("位置单元维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("位置单元维护修改前数据是" + tRmLocationService.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TRmLocation tRmLocation = tRmLocationService.modify(model[i]);
                LOGGER.info("位置单元维护修改后数据是" + tRmLocation.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "位置单元维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }
    @ApiOperation(value="位置单元下拉框", notes="查询id和name")
    @GetMapping(value = "/getidandname/")
    public RESTfulResult getNameAndId(){
        List<TRmLocation> tRmLocation = tRmLocationService.getNameAndId();
        return success(tRmLocation);
    }
}
