package com.steerinfo.ems.trmstation.controller;

import com.steerinfo.auth.utils.JwtUtil;
import com.steerinfo.ems.trmstation.model.TRmStation;
import com.steerinfo.ems.trmstation.service.ITRmStationService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * TRmStation RESTful接口：
 * @author 王金涛
 * @version 1.0-SNAPSHORT 2019-10-11
 * 类描述
 * 修订历史：
 * 日期：2019-10-11
 * 作者：王金涛
 * 参考：
 * 描述：TRmStation RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/trmstations")
public class TRmStationController extends BaseRESTfulController {

    private final static Logger LOGGER = LoggerFactory.getLogger(TRmStationController.class);

    @Autowired
    ITRmStationService tRmStationService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmstation:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmStation> list = tRmStationService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("trmstation:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TRmStation> list = tRmStationService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TRmStation对象创建")
    @ApiImplicitParam(name = "tRmStation", value = "详细实体tRmStation", required = true, dataType = "TRmStation")
    //@RequiresPermissions("trmstation:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TRmStation model){
        if(tRmStationService.getById(model.getId()) != null){
            return failed(null, "id已存在，不能新增");
        } else {
        	if(model.getSortseq()==null){
        		model.setSortseq(tRmStationService.getMaxSortSeq() + 1);
        	}
            model.setCreateman(JwtUtil.getUseridByToken());
            TRmStation tRmStation = tRmStationService.add(model);
            return success(tRmStation);
        }
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmstation:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TRmStation tRmStation = tRmStationService.getById(id);
        return success(tRmStation);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的tRmStation信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "tRmStation", value = "详细实体tRmStation", required = true, dataType = "TRmStation")
    })
    //@RequiresPermissions("trmstation:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TRmStation model){
        LOGGER.info("站所维护表修改前数据是" + tRmStationService.getById(id).toString());
        model.setId(id);
        model.setUpdateman(JwtUtil.getUseridByToken());
        TRmStation tRmStation = tRmStationService.modify(model);
        LOGGER.info("站所维护表修改后数据是" + tRmStation.toString());
        return success(tRmStation);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("trmstation:delete")
    @DeleteMapping(value = "/{id}")
    public RESTfulResult delete(@PathVariable String id){
        String[] array = id.split(",");
        if(array.length >= 1) {
            int failnum = 0;
            String failmsg = "";
            for (int i = 0; i < array.length; i++) {
                try {
                    LOGGER.info("站所维护表要删除的第" + (i + 1) + "条数据是" + tRmStationService.getById(array[i]).toString());
                    tRmStationService.delete(array[i]);
                }
                catch(Exception e){
                    failnum++;
                    failmsg += "站所维护表删除第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + array[i];
                }
            }
            if(failnum > 0){
                LOGGER.info(JwtUtil.getUseridByToken() + "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
                return failed(null, "本次共删除" + array.length + "条数据,其中成功" + (array.length - failnum) + "条,失败" + failnum + "条,失败的数据id:" + array.toString());
            }
            return success();
        }
        else {
            return failed(null, "请输入要删除的id");
        }
    }

    @ApiOperation(value="批量更新详细信息", notes="根据传过来的tRmStation数组信息来更新详细信息")
    @ApiImplicitParam(name = "tRmStation", value = "详细实体tRmStation", required = true, dataType = "TRmStation")
    //@RequiresPermissions("trmworkproc:update")
    @PutMapping(value = "/", produces  = "application/json;charset=UTF-8")
    public RESTfulResult batchupdate(@RequestBody TRmStation[] model){
        int failnum = 0;
        String failmsg = "";
        List<String> errorid = new ArrayList<>();
        for(int i = 0;i < model.length;i++){
            try {
                LOGGER.info("站所信息维护修改的第" + (i + 1) + "条数据:");
                LOGGER.info("站所信息维护修改前数据是" + tRmStationService.getById(model[i].getId()).toString());
                model[i].setUpdateman(JwtUtil.getUseridByToken());
                TRmStation tRmStation = tRmStationService.modify(model[i]);
                LOGGER.info("站所信息维护修改后数据是" + tRmStation.toString());
            }
            catch(Exception e){
                failnum++;
                errorid.add(model[i].getId());
                failmsg += "站所信息维护修改在第" + (i + 1) + "条数据出现错误,错误原因是" + e + ",id是" + model[i].getId();
            }
        }
        if(failnum > 0){
            LOGGER.info("本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条,失败原因:" + failmsg);
            return success(errorid,"51","本次共修改" + model.length + "条数据,其中成功" + (model.length - failnum) + "条,失败" + failnum + "条");
        }
        return success();
    }

    @ApiOperation(value="站所信息下拉框", notes="查询id和name")
    @GetMapping(value = "/getidandname/")
    public RESTfulResult getNameAndId(){
        List<TRmStation> tRmStation = tRmStationService.getNameAndId();
        return success(tRmStation);
    }
}
