package com.steerinfo.task.taskconf.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.task.SpringCronTask;
import com.steerinfo.task.taskconf.model.TaskConf;
import com.steerinfo.task.taskconf.service.ITaskConfService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.HashMap;

/**
 * TaskConf RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2019-11-04 07:44
 * 类描述
 * 修订历史：
 * 日期：2019-11-04
 * 作者：generator
 * 参考：
 * 描述：TaskConf RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/taskconfs")
public class TaskConfController extends BaseRESTfulController {

    @Autowired
    ITaskConfService taskConfService;
    
    @Autowired
	SpringCronTask springCronTask;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("taskconf:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TaskConf> list = taskConfService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("taskconf:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<TaskConf> list = taskConfService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据TaskConf对象创建")
    @ApiImplicitParam(name = "taskConf", value = "详细实体taskConf", required = true, dataType = "TaskConf")
    //@RequiresPermissions("taskconf:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute TaskConf model){
        TaskConf taskConf = taskConfService.add(model);
        return success(taskConf);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("taskconf:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        TaskConf taskConf = taskConfService.getById(id);
        return success(taskConf);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的taskConf信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "taskConf", value = "详细实体taskConf", required = true, dataType = "TaskConf")
    })
    //@RequiresPermissions("taskconf:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody TaskConf model){
    	// model.setId(id);
        model.setId("1");
        TaskConf taskConf = taskConfService.modify(model);
        return success(taskConf);
    }
    
    @ApiOperation(value="更新任务信息", notes="现阶段主要修改时间cron")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "cron", value = "定时任务执行时间", required = true)
    })
	@PutMapping(value = "/update/")
    public RESTfulResult update(String taskId, String cron){
		if (cron == null || cron.isEmpty()) {
			return failed(null, "请传入时间");
		}
		taskId = "1";
		TaskConf taskConf = new TaskConf();
		taskConf.setId(taskId);
		taskConf.setCron(cron);
		taskConfService.modify(taskConf);
		springCronTask.updateTask(taskId);
        return success();
    }

//    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
//    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
//    //@RequiresPermissions("taskconf:delete")
//    @DeleteMapping(value = "/{id}")
//    public RESTfulResult delete(@PathVariable String id){
//        if (StringUtils.indexOf(id, ",") == -1) {
//            taskConfService.delete(id);
//        } else {
//            taskConfService.delete(Arrays.asList(id.split(",")));
//        }
//        return success();
//    }
}
